/* $Id: globus_i_replica_catalog.c,v 1.4 2001/08/30 21:57:49 meder Exp $ */


#include "globus_replica_catalog.h"
#include "globus_replica_catalog_sets.h"
#include "globus_i_replica_catalog.h"

int
globus_i_replica_catalog_collection_handleattr_compare(
    globus_replica_catalog_collection_handleattr_t *    attrs1,
    globus_replica_catalog_collection_handleattr_t *    attrs2)
{
    /* needs to be filled in */
    return 0;
}

globus_result_t
globus_i_replica_catalog_location_get_url(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          filename,
    char **                                         url)
{
    char *                                          attribute_names[2];
    char *                                          name;
    char **                                         values;
    struct globus_replica_catalog_attribute_set_s   attributes;
    globus_result_t                                 rc;

    if(handle == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "NULL handle");
	goto return_result;
    }

    if(location_name == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "NULL location name");
	goto return_result;
    }

    if(filename == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "NULL file name");
	goto return_result;
    }

    
    attribute_names[0] = "uc";
    attribute_names[1] = GLOBUS_NULL;

    rc = globus_replica_catalog_location_list_attributes(
	handle,
	location_name,
	attribute_names,
	&attributes);

    if(rc != GLOBUS_SUCCESS)
    {
	goto return_result;
    }

    globus_replica_catalog_attribute_set_first(&attributes);

    if(globus_replica_catalog_attribute_set_more(&attributes))
    {
	globus_replica_catalog_attribute_set_get_values(&attributes,
							&values);
	globus_replica_catalog_attribute_set_get_name(&attributes,
						      &name);
	
	globus_assert(name != GLOBUS_NULL &&
		      values != GLOBUS_NULL &&
		      values[0] != GLOBUS_NULL);
    }
    else
    {
	rc = GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Unable to obtain URL");
	goto destroy_attributes;
    }

    *url = globus_libc_malloc(strlen(values[0]) +
			      strlen(filename) + 2);
    
    if(*url == GLOBUS_NULL)
    {
	rc = GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Unable to allocate internal data structure");
	goto free_values;	
    }

    strcpy(*url, values[0]);

    if((*url)[strlen(values[0])-1] != '/')
    {
	strcat(*url,"/");
    }

    strcat(*url, filename);
    
    rc = GLOBUS_SUCCESS;

free_values:
    globus_replica_catalog_attribute_set_free_values(values);
destroy_attributes:
    globus_replica_catalog_attribute_set_destroy(&attributes);
return_result:
    return rc;
}

globus_result_t
globus_i_replica_catalog_location_delete_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char **                                         filenames)
{
    char *                                          fail;
    static int                                      iteration=0;
    int                                             failure_iteration;
    
    fail = getenv("FAIL_LOCATION_DELETE_FILENAMES");

    iteration++;
    
    if(fail == GLOBUS_NULL ||
       (failure_iteration = atoi(fail)) != iteration)
    {
	return globus_replica_catalog_location_delete_filenames(
	    handle,
	    location_name,
	    filenames);
    }
    else
    {
	return GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Induced failure");
    }
}

globus_result_t
globus_i_replica_catalog_rollback_info_update_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          logicalfile_name,
    char *                                          state,
    char *                                          ranges)
{
    char *                                          fail;
    static int                                      iteration=0;
    int                                             failure_iteration;
    
    fail = getenv("FAIL_ROLLBACK_INFO_UPDATE");

    iteration++;
    
    if(fail == GLOBUS_NULL ||
       (failure_iteration = atoi(fail)) != iteration)
    {
	return globus_i_replica_catalog_rollback_info_update(handle,
							     location_name,
							     logicalfile_name,
							     state,
							     ranges);
    }
    else
    {
	
	return GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Induced failure");
    }
}


globus_result_t
globus_i_replica_catalog_rollback_info_delete_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          logicalfile_name)
{
    char *                                          fail;
    static int                                      iteration=0;
    int                                             failure_iteration;
    
    fail = getenv("FAIL_ROLLBACK_INFO_DELETE");

    iteration++;
    
    if(fail == GLOBUS_NULL ||
       (failure_iteration = atoi(fail)) != iteration)
    {
	return globus_i_replica_catalog_rollback_info_delete(handle,
							     location_name,
							     logicalfile_name);
    }
    else
    {
	
	return GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Induced failure");
    }
}

globus_result_t
globus_i_replica_catalog_location_add_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char **                                         filenames,
    globus_bool_t                                   add)
{
    char *                                          fail;
    static int                                      iteration=0;
    int                                             failure_iteration;
    
    fail = getenv("FAIL_LOCATION_ADD_FILENAMES");

    iteration++;
    
    if(fail == GLOBUS_NULL ||
       (failure_iteration = atoi(fail)) != iteration)
    {
	return globus_replica_catalog_location_add_filenames(handle,
							     location_name,
							     filenames,
							     add);
    }
    else
    {
	
	return GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Induced failure");
    }
}

globus_result_t
globus_i_replica_catalog_collection_add_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char **                                         filenames,
    globus_bool_t                                   add)
{
    char *                                          fail;
    static int                                      iteration=0;
    int                                             failure_iteration;
    
    fail = getenv("FAIL_COLLECTION_ADD_FILENAMES");

    iteration++;
    
    if(fail == GLOBUS_NULL ||
       (failure_iteration = atoi(fail)) != iteration)
    {
	return globus_replica_catalog_collection_add_filenames(handle,
							       filenames,
							       add);
    }
    else
    {
	
	return GLOBUS_REPLICA_CATALOG_ERROR( 
	    GLOBUS_NULL,
	    "Induced failure");
    }
}





